Stack

Struct Stack 

pub struct Stack(/* private fields */);
The STD network stack implementation.

This uses the standard library’s networking types under the hood, wrapped in async-io’s Async type for async support.

The type is Copy and Clone, so it can be easily passed around.

Implementations

Source

impl Stack

pub const fn new() -> Self

Create a new STD network stack instance.